"""
Definition of the DMTF MOF Schema to be used in this testsuite and the
code to install it if not already installed and unzipped.

The version defined below will be installed in the directory SCHEMA_DIR
(testsuite/schema) if that directory is empty or the file does not exist.

Otherwise, the tests will be executed with that defined version of the
schema.

NOTE: The zip expansion is NOT committed to git, just the original zip file.

To change the schema used:

1. Change the following variables to match the DMTF schema to be used
    (MOF_ZIP_BN, MOF_ZIP_URL, SCHEMA_MOF_BN)
    NOTE: The BN, URL, and MOF_BN will change between versions. The V2xxx in the
        URL must be updated and the version number in both MOF_ZIP_BN and
        SCHEMA_MOF_BIN updated.
    See the page http://www.dmtf.org/standards/cim if there are issues
    downloading a particular schema version.

2. Delete the SCHEMA_DIR (testsuite/schema). Be sure to delete the directory
   to be sure the new schema gets downloaded and correctly expanded.

3. Execute testsuite/test_mof_compiler.py. This should cause the new schema
   to be downloaded and expanded as part of the test.

4. The first test should generate an error if the values for total number of
   classes or qualifiers have changed. Modify the  variables below to define
   the correct numbers and re-execute test_mof_compiler.
   NOTE: We are keeping some history of the counts for previous versions of
   the schema (see the comments at the end of this file)
"""

import os
from zipfile import ZipFile
import six

if six.PY2:
    # pylint: disable=wrong-import-order
    from urllib2 import urlopen
else:
    # pylint: disable=wrong-import-order
    from urllib.request import urlopen

# Change the following variables when a new version of the CIM Schema is used
# and remove the SCHEMA_DIR directory
# This defines the version and the location of the schema zip file on the
# DMTF web site.
# See the page http://www.dmtf.org/standards/cim if there are issues
# downloading a particular version.

# Location of the schema for use by test_mof_compiler.
# This should not change unless you intend to use another schema directory
SCRIPT_DIR = os.path.dirname(__file__)
SCHEMA_DIR = os.path.join(SCRIPT_DIR, 'schema')
SCHEMA_MOF_DIR = os.path.join(SCHEMA_DIR, 'mof')

DMTF_SCHEMA_VERSION = 'cim_schema_2.49.0'
MOF_ZIP_BN = DMTF_SCHEMA_VERSION + 'Final-MOFs.zip'
MOF_ZIP_URL = 'http://www.dmtf.org/standards/cim/cim_schema_v2490/' + MOF_ZIP_BN
SCHEMA_MOF_BN = DMTF_SCHEMA_VERSION + '.mof'

# DMTF Schema zip filename and mof filename
MOF_ZIP_FN = os.path.join(SCHEMA_DIR, MOF_ZIP_BN)
SCHEMA_MOF_FN = os.path.join(SCHEMA_MOF_DIR, SCHEMA_MOF_BN)

# Expected total of qualifiers and classes in the DMTF Schema.
# These may change for each schema release and will need to be manually
# modified here to correctly execute the tests.
# 2.49.0
TOTAL_QUALIFIERS = 70
TOTAL_CLASSES = 1631

# Qualifier and Class counts for previous DMTF schema.
# 2.48.0
# TOTAL_QUALIFIERS = 70
# TOTAL_CLASSES = 1630


def install_dmtf_schema():
    """
    Install the DMTF schema if it is not already installed.  All the
    definitions of the installation are in the module variables.
    The user of ths should need
    """
    first = True

    if not os.path.isdir(SCHEMA_DIR):
        if first:
            print("")
            first = False
        print("Creating directory for CIM Schema archive: %s" % SCHEMA_DIR)
        os.mkdir(SCHEMA_DIR)

    if not os.path.isfile(MOF_ZIP_FN):
        if first:
            print("")
            first = False
        print("Downloading CIM Schema archive from: %s" % MOF_ZIP_URL)
        ufo = urlopen(MOF_ZIP_URL)
        with open(MOF_ZIP_FN, 'w') as fp:
            for data in ufo:
                fp.write(data)

    if not os.path.isdir(SCHEMA_MOF_DIR):
        if first:
            print("")
            first = False
        print("Creating directory for CIM Schema MOF files: %s" %
              SCHEMA_MOF_DIR)
        os.mkdir(SCHEMA_MOF_DIR)

    if not os.path.isfile(SCHEMA_MOF_FN):
        if first:
            print("")
            first = False
        print("Unpacking CIM Schema archive: %s" % MOF_ZIP_FN)
        try:
            zfp = ZipFile(MOF_ZIP_FN, 'r')
            nlist = zfp.namelist()
            for file_ in nlist:
                dfile = os.path.join(SCHEMA_MOF_DIR, file_)
                if dfile[-1] == '/':
                    if not os.path.exists(dfile):
                        os.mkdir(dfile)
                else:
                    with open(dfile, 'w+b') as dfp:
                        dfp.write(zfp.read(file_))
        finally:
            zfp.close()



#!/usr/bin/env python
"""
Python script for testing the internal _uprint() function with destination
stdout, in a Python environment that is as close as possible to what a pywbem
user uses.

Specifically, because py.test captures stdout, this program should not run
under py.test (unless there is a separate shell in between this program and
py.test).

If the _uprint() function succeeds, the script exits with exit code 0, and a
test string has been written to stdout.

Otherwise, the script exits with an exit code other than 0, and an error
message has been written to stderr (and possibly stdout).

This test script can be invoked with a command line argument that determines
the test case that is run:

* run_uprint.py small - A small string of selected unicode characters
* run_uprint.py ucs2 - All Unicode characters in UCS-2 (U+0000 to U+FFFF)
* run_uprint.py all - All Unicode characters up to the max character supported
  by the Python build that is used. A wide build supports up to U+10FFFF. A
  narrow build supports up to U+FFFF. If the Python build supports less than
  U+10FFFF, an info message is written to stderr, but this does not cause the
  test to fail.
"""

from __future__ import absolute_import, print_function
import sys
import locale
import six
from pywbem_mock._wbemconnection_mock import _uprint


def main():
    """Main routine"""

    if len(sys.argv) > 1:
        mode = sys.argv[1]  # 'small', 'ucs2', 'all'
    else:
        mode = 'small'

    print("Debug: mode=%s; sys.stdout: isatty=%r, encoding=%r" %
          (mode, sys.stdout.isatty(), getattr(sys.stdout, 'encoding', None)),
          file=sys.stderr)
    print("Debug: sys.getfilesystemencoding=%r locale.getpreferredencoding=%r" %
          (sys.getfilesystemencoding(), locale.getpreferredencoding()),
          file=sys.stderr)

    if mode == 'small':
        test_string = u'\u212b \u0420 \u043e \u0441 \u0441 \u0438 \u044f \u00e0'
        _uprint(None, test_string)
    elif mode == 'ucs2':
        for cp in six.moves.xrange(0, 0xFFFF):
            test_string = six.unichr(cp)
            _uprint(None, test_string)
    elif mode == 'all':
        test_string = u''
        if sys.maxunicode < 0x10FFFF:
            print("Info: Testing Unicode range of only up to U+%.6X "
                  "(out of U+10FFFF)" % sys.maxunicode,
                  file=sys.stderr)
        for cp in six.moves.xrange(0, sys.maxunicode):
            test_string = six.unichr(cp)
            _uprint(None, test_string)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
"""
    Test case-insensitive dictionary implementation.
"""

from __future__ import absolute_import

import unittest
import warnings
import six

import pytest
import pytest_extensions

from pywbem.cim_obj import NocaseDict


class NonCompare(object):
    # pylint: disable=too-few-public-methods
    """Class that raises TypeError when comparing for equality or hashing."""

    def __eq__(self, other):
        raise TypeError("Cannot compare %s to %s" % (type(self), type(other)))

    def __ne__(self, other):
        raise TypeError("Cannot compare %s to %s" % (type(self), type(other)))

    def __hash__(self):
        raise TypeError("Cannot hash %s" % type(self))


class TestInit(unittest.TestCase):
    """Test initialization"""
    def test_all(self):
        """Test all init options"""
        # Empty

        dic = NocaseDict()
        self.assertTrue(len(dic) == 0)

        dic = NocaseDict(None)
        self.assertTrue(len(dic) == 0)

        dic = NocaseDict(list())
        self.assertTrue(len(dic) == 0)

        dic = NocaseDict(tuple())
        self.assertTrue(len(dic) == 0)

        dic = NocaseDict(dict())
        self.assertTrue(len(dic) == 0)

        dic = NocaseDict(dic)
        self.assertTrue(len(dic) == 0)

        # Initialise from iterable

        dic = NocaseDict([('Dog', 'Cat'), ('Budgie', 'Fish')])
        self.assertTrue(len(dic) == 2)
        self.assertTrue(dic['Dog'] == 'Cat' and dic['Budgie'] == 'Fish')

        dic = NocaseDict((('Dog', 'Cat'), ('Budgie', 'Fish')))
        self.assertTrue(len(dic) == 2)
        self.assertTrue(dic['Dog'] == 'Cat' and dic['Budgie'] == 'Fish')

        # Initialise from dictionary

        with pytest.warns(UserWarning) as rec_warnings:
            dic = NocaseDict({'Dog': 'Cat', 'Budgie': 'Fish'})
        assert len(rec_warnings) == 1
        self.assertTrue(len(dic) == 2)
        self.assertTrue(dic['Dog'] == 'Cat' and dic['Budgie'] == 'Fish')

        # Initialise from kwargs

        with pytest.warns(UserWarning) as rec_warnings:
            dic = NocaseDict(Dog='Cat', Budgie='Fish')
        assert len(rec_warnings) == 1
        self.assertTrue(len(dic) == 2)
        self.assertTrue(dic['Dog'] == 'Cat' and dic['Budgie'] == 'Fish')

        # Initialise from iterable and kwargs

        dic = NocaseDict([('Dog', 'Cat'), ], Budgie='Fish')
        self.assertTrue(len(dic) == 2)
        self.assertTrue(dic['Dog'] == 'Cat' and dic['Budgie'] == 'Fish')

        dic = NocaseDict((('Dog', 'Cat'),), Budgie='Fish')
        self.assertTrue(len(dic) == 2)
        self.assertTrue(dic['Dog'] == 'Cat' and dic['Budgie'] == 'Fish')

        # Initialise from dictionary and kwargs

        dic = NocaseDict({'Dog': 'Cat'}, Budgie='Fish')
        self.assertTrue(len(dic) == 2)
        self.assertTrue(dic['Dog'] == 'Cat' and dic['Budgie'] == 'Fish')

        # Initialise from unsupported object type

        try:
            dic = NocaseDict('illegal')
        except TypeError:
            pass
        else:
            self.fail("TypeError was unexpectedly not thrown.")

        # Initialise with too many positional arguments

        try:
            dic = NocaseDict(list(), list())
        except TypeError:
            pass
        else:
            self.fail("TypeError was unexpectedly not thrown.")


class BaseTest(unittest.TestCase):
    """Base class for following unit test. Does common setup which
       creates a NoCaseDict.
    """
    def setUp(self):
        """unittest setUp creates NoCaseDict"""

        self.dic = NocaseDict()
        self.dic['Dog'] = 'Cat'
        self.dic['Budgie'] = 'Fish'

        self.order_tuples = (
            ('Dog', 'Cat'),
            ('Budgie', 'Fish'),
            ('Ham', 'Jam'),
            ('Sofi', 'Blue'),
            ('Gabi', 'Red'),
        )


class TestGetitem(BaseTest):
    """Tests for getitem"""
    def test_all(self):
        """All tests"""
        self.assertTrue(self.dic['dog'] == 'Cat')
        self.assertTrue(self.dic['DOG'] == 'Cat')

        try:
            self.dic['notfound']
        except KeyError:
            pass
        else:
            self.fail("KeyError was unexpectedly not thrown.")


class TestLen(BaseTest):
    """Tests for len of dict"""
    def test_all(self):
        """Test method"""
        self.assertTrue(len(self.dic) == 2)


class TestSetitem(BaseTest):
    """Test setting items"""
    def test_all(self):
        """All setitem tests"""
        self.dic['DOG'] = 'Kitten'
        self.assertTrue(self.dic['DOG'] == 'Kitten')
        self.assertTrue(self.dic['Dog'] == 'Kitten')
        self.assertTrue(self.dic['dog'] == 'Kitten')

        # Check that using a non-string key raises an exception

        try:
            self.dic[1234] = '1234'
        except TypeError:
            pass
        else:
            self.fail('TypeError expected')


class TestDelitem(BaseTest):
    """Class for del items from dictionary"""
    def test_all(self):
        """All tests"""
        del self.dic['DOG']
        del self.dic['budgie']
        self.assertTrue(self.dic.keys() == [])

        try:
            del self.dic['notfound']
        except KeyError:
            pass
        else:
            self.fail("KeyError was unexpectedly not thrown.")


class TestHasKey(BaseTest):
    """Class to test haskey on dict"""
    def test_all(self):
        """Method to test haskey"""
        self.assertTrue('DOG' in self.dic)
        self.assertTrue('budgie' in self.dic)

        try:
            1234 in self.dic
        except TypeError:
            pass
        else:
            self.fail('TypeError expected')


class TestKeys(BaseTest):
    """Class for TestKeys method"""
    def test_all(self):
        """All tests in single method"""
        keys = self.dic.keys()
        animals = ['Budgie', 'Dog']
        for ani in animals:
            self.assertTrue(ani in keys)
            keys.remove(ani)
        self.assertTrue(keys == [])


class TestValues(BaseTest):
    """Class for values tests"""
    def test_all(self):
        """Test all for TestValues"""
        values = self.dic.values()
        animals = ['Cat', 'Fish']
        for ani in animals:
            self.assertTrue(ani in values)
            values.remove(ani)
        self.assertTrue(values == [])


class TestItems(BaseTest):
    """Class for Test items"""
    def test_all(self):
        """All tests for item"""
        items = self.dic.items()
        animals = [('Dog', 'Cat'), ('Budgie', 'Fish')]
        for ani in animals:
            self.assertTrue(ani in items)
            items.remove(ani)
        self.assertTrue(items == [])


class TestClear(BaseTest):
    """Class for dict clear method"""
    def test_all(self):
        """All clear method tests"""
        self.dic.clear()
        self.assertTrue(len(self.dic) == 0)


class TestUpdate(BaseTest):
    """Class for test update method"""
    def test_all(self):
        """All methods for TestUpdate"""
        self.dic.clear()
        self.dic.update({'Chicken': 'Ham'})
        self.assertTrue(self.dic.keys() == ['Chicken'])
        self.assertTrue(self.dic.values() == ['Ham'])
        self.dic.clear()
        self.dic.update({'Chicken': 'Ham'}, {'Dog': 'Cat'})
        keys = self.dic.keys()
        vals = self.dic.values()
        keys = list(keys)
        vals = list(vals)
        keys.sort()
        vals.sort()
        self.assertTrue(keys == ['Chicken', 'Dog'])
        self.assertTrue(vals == ['Cat', 'Ham'])
        self.dic.update([('Chicken', 'Egg')], {'Fish': 'Eel'})
        self.assertTrue(self.dic['chicken'] == 'Egg')
        self.assertTrue(self.dic['fish'] == 'Eel')
        self.dic.update({'Fish': 'Salmon'}, Cow='Beef')
        self.assertTrue(self.dic['fish'] == 'Salmon')
        self.assertTrue(self.dic['Cow'] == 'Beef')
        self.assertTrue(self.dic['COW'] == 'Beef')
        self.assertTrue(self.dic['cow'] == 'Beef')


class TestCopy(BaseTest):
    """Class to test dict copy"""
    def test_all(self):
        """All tests for dict copy"""
        cp = self.dic.copy()
        self.assertEqual(cp, self.dic)
        self.assertTrue(isinstance(cp, NocaseDict))
        cp['Dog'] = 'Kitten'
        self.assertTrue(self.dic['Dog'] == 'Cat')
        self.assertTrue(cp['Dog'] == 'Kitten')


class TestGet(BaseTest):
    """Class to test get method"""
    def test_all(self):
        """Test get method"""
        self.assertTrue(self.dic.get('Dog', 'Chicken') == 'Cat')
        self.assertTrue(self.dic.get('Ningaui') is None)
        self.assertTrue(self.dic.get('Ningaui', 'Chicken') == 'Chicken')


class TestSetDefault(BaseTest):
    """Class for setdefault test methods"""
    def test_all(self):
        """All tests for setdefault of dict"""
        self.dic.setdefault('Dog', 'Kitten')
        self.assertTrue(self.dic['Dog'] == 'Cat')
        self.dic.setdefault('Ningaui', 'Chicken')
        self.assertTrue(self.dic['Ningaui'] == 'Chicken')


class TestPopItem(BaseTest):
    """Class for PopItem"""
    def test_all(self):
        """This test does nothing"""
        pass


TESTCASES_NOCASEDICT_EQUAL_HASH = [

    # Each testcase tuple has these items:
    # * desc: Short testcase description.
    # * kwargs: Input arguments for test function, as a dict:
    #   * obj1: CIMInstanceName object #1 to use.
    #   * obj2: CIMInstanceName object #2 to use.
    #   * exp_obj_equal: Expected equality of the objects.
    # * exp_exc_types: Expected exception type(s), or None.
    # * exp_warn_types: Expected warning type(s), or None.
    # * condition: Boolean condition for testcase to run, or 'pdb' for debugger

    (
        "Empty dictionary",
        dict(
            obj1=NocaseDict([]),
            obj2=NocaseDict([]),
            exp_obj_equal=True,
        ),
        None, None, True
    ),
    (
        "One item, keys and values equal",
        dict(
            obj1=NocaseDict([('k1', 'v1')]),
            obj2=NocaseDict([('k1', 'v1')]),
            exp_obj_equal=True,
        ),
        None, None, True
    ),
    (
        "One item, keys equal, values different",
        dict(
            obj1=NocaseDict([('k1', 'v1')]),
            obj2=NocaseDict([('k1', 'v1_x')]),
            exp_obj_equal=False,
        ),
        None, None, True
    ),
    (
        "One item, keys different, values equal",
        dict(
            obj1=NocaseDict([('k1', 'v1')]),
            obj2=NocaseDict([('k2', 'v1')]),
            exp_obj_equal=False,
        ),
        None, None, True
    ),
    (
        "One item, keys equal, values both None",
        dict(
            obj1=NocaseDict([('k1', None)]),
            obj2=NocaseDict([('k1', None)]),
            exp_obj_equal=True,
        ),
        None, None, True
    ),
    (
        "One item, keys different lexical case, values equal",
        dict(
            obj1=NocaseDict([('K1', 'v1')]),
            obj2=NocaseDict([('k1', 'v1')]),
            exp_obj_equal=True,
        ),
        None, None, True
    ),
    (
        "Two equal items, in same order",
        dict(
            obj1=NocaseDict([('k1', 'v1'), ('k2', 'v2')]),
            obj2=NocaseDict([('k1', 'v1'), ('k2', 'v2')]),
            exp_obj_equal=True,
        ),
        None, None, True
    ),
    (
        "Two items, keys different lexical case, in same order",
        dict(
            obj1=NocaseDict([('K1', 'v1'), ('k2', 'v2')]),
            obj2=NocaseDict([('k1', 'v1'), ('K2', 'v2')]),
            exp_obj_equal=True,
        ),
        None, None, True
    ),
    (
        "Two equal items, in different order",
        dict(
            obj1=NocaseDict([('k1', 'v1'), ('k2', 'v2')]),
            obj2=NocaseDict([('k2', 'v2'), ('k1', 'v1')]),
            exp_obj_equal=True,
        ),
        None, None, True
    ),
    (
        "Two items, keys different lexical case, in different order",
        dict(
            obj1=NocaseDict([('k1', 'v1'), ('K2', 'v2')]),
            obj2=NocaseDict([('k2', 'v2'), ('K1', 'v1')]),
            exp_obj_equal=True,
        ),
        None, None, True
    ),
    (
        "Comparing unicode value with binary value",
        dict(
            obj1=NocaseDict([('k1', b'v1')]),
            obj2=NocaseDict([('k2', u'v2')]),
            exp_obj_equal=False,
        ),
        None, None, True
    ),
    (
        "Matching unicode key with string key",
        dict(
            obj1=NocaseDict([('k1', 'v1')]),
            obj2=NocaseDict([(u'k2', 'v2')]),
            exp_obj_equal=False,
        ),
        None, None, True
    ),
    (
        "Higher key missing",
        dict(
            obj1=NocaseDict([('Budgie', 'Fish'), ('Dog', 'Cat')]),
            obj2=NocaseDict([('Budgie', 'Fish')]),
            exp_obj_equal=False,
        ),
        None, None, True
    ),
    (
        "Lower key missing",
        dict(
            obj1=NocaseDict([('Budgie', 'Fish'), ('Dog', 'Cat')]),
            obj2=NocaseDict([('Dog', 'Cat')]),
            exp_obj_equal=False,
        ),
        None, None, True
    ),
    (
        "First non-matching key is less. But longer size!",
        dict(
            obj1=NocaseDict([('Budgie', 'Fish'), ('Dog', 'Cat')]),
            obj2=NocaseDict([('Budgie', 'Fish'), ('Curly', 'Snake'),
                             ('Cozy', 'Dog')]),
            exp_obj_equal=False,
        ),
        None, None, True
    ),
    (
        "Only non-matching keys that are less. But longer size!",
        dict(
            obj1=NocaseDict([('Budgie', 'Fish'), ('Dog', 'Cat')]),
            obj2=NocaseDict([('Alf', 'F'), ('Anton', 'S'), ('Aussie', 'D')]),
            exp_obj_equal=False,
        ),
        None, None, True
    ),
    (
        "First non-matching key is greater. But shorter size!",
        dict(
            obj1=NocaseDict([('Budgie', 'Fish'), ('Dog', 'Cat')]),
            obj2=NocaseDict([('Budgio', 'Fish')]),
            exp_obj_equal=False,
        ),
        None, None, True
    ),
    (
        "Only non-matching keys that are greater. But shorter size!",
        dict(
            obj1=NocaseDict([('Budgie', 'Fish'), ('Dog', 'Cat')]),
            obj2=NocaseDict([('Zoe', 'F')]),
            exp_obj_equal=False,
        ),
        None, None, True
    ),
    (
        "Same size. First non-matching key is less",
        dict(
            obj1=NocaseDict([('Budgie', 'Fish'), ('Dog', 'Cat')]),
            obj2=NocaseDict([('Budgie', 'Fish'), ('Curly', 'Snake')]),
            exp_obj_equal=False,
        ),
        None, None, True
    ),
    (
        "Same size. Only non-matching keys that are less",
        dict(
            obj1=NocaseDict([('Budgie', 'Fish'), ('Dog', 'Cat')]),
            obj2=NocaseDict([('Alf', 'F'), ('Anton', 'S')]),
            exp_obj_equal=False,
        ),
        None, None, True
    ),
    (
        "Same size. Only non-matching keys that are greater",
        dict(
            obj1=NocaseDict([('Budgie', 'Fish'), ('Dog', 'Cat')]),
            obj2=NocaseDict([('Zoe', 'F'), ('Zulu', 'S')]),
            exp_obj_equal=False,
        ),
        None, None, True
    ),
    (
        "Same size, only matching keys. First non-matching value is less",
        dict(
            obj1=NocaseDict([('Budgie', 'Fish'), ('Dog', 'Cat')]),
            obj2=NocaseDict([('Budgie', 'Fish'), ('Dog', 'Car')]),
            exp_obj_equal=False,
        ),
        None, None, True
    ),
    (
        "Same size, only matching keys. First non-matching value is greater",
        dict(
            obj1=NocaseDict([('Budgie', 'Fish'), ('Dog', 'Cat')]),
            obj2=NocaseDict([('Budgie', 'Fish'), ('Dog', 'Caz')]),
            exp_obj_equal=False,
        ),
        None, None, True
    ),
]

TESTCASES_NOCASEDICT_EQUAL = [

    # Each testcase tuple has these items:
    # * desc: Short testcase description.
    # * kwargs: Input arguments for test function, as a dict:
    #   * obj1: CIMInstanceName object #1 to use.
    #   * obj2: CIMInstanceName object #2 to use.
    #   * exp_obj_equal: Expected equality of the objects.
    # * exp_exc_types: Expected exception type(s), or None.
    # * exp_warn_types: Expected warning type(s), or None.
    # * condition: Boolean condition for testcase to run, or 'pdb' for debugger

    (
        "A value raises TypeError when compared (and equal still succeeds)",
        dict(
            obj1=NocaseDict([('Budgie', 'Fish'), ('Dog', 'Cat')]),
            obj2=NocaseDict([('Budgie', NonCompare()), ('Dog', 'Cat')]),
            exp_obj_equal=False,
        ),
        None, None, True
    ),
]

TESTCASES_NOCASEDICT_HASH = [

    # Each testcase tuple has these items:
    # * desc: Short testcase description.
    # * kwargs: Input arguments for test function, as a dict:
    #   * obj1: CIMInstanceName object #1 to use.
    #   * obj2: CIMInstanceName object #2 to use.
    #   * exp_obj_equal: Expected equality of the objects.
    # * exp_exc_types: Expected exception type(s), or None.
    # * exp_warn_types: Expected warning type(s), or None.
    # * condition: Boolean condition for testcase to run, or 'pdb' for debugger

    (
        "A value raises TypeError when compared (and hash fails)",
        dict(
            obj1=NocaseDict([('Budgie', 'Fish'), ('Dog', 'Cat')]),
            obj2=NocaseDict([('Budgie', NonCompare()), ('Dog', 'Cat')]),
            exp_obj_equal=False,
        ),
        TypeError, None, True
    ),
]


@pytest.mark.parametrize(
    "desc, kwargs, exp_exc_types, exp_warn_types, condition",
    TESTCASES_NOCASEDICT_EQUAL_HASH + TESTCASES_NOCASEDICT_EQUAL)
@pytest_extensions.test_function
def test_NocaseDict_equal(
        desc, kwargs, exp_exc_types, exp_warn_types, condition):
    """
    All test cases for NocaseDict.__eq__().
    """

    obj1 = kwargs['obj1']
    obj2 = kwargs['obj2']

    # Double check they are different objects
    assert id(obj1) != id(obj2)

    # The code to be tested
    eq1 = (obj1 == obj2)
    eq2 = (obj2 == obj1)
    ne1 = (obj1 != obj2)
    ne2 = (obj2 != obj1)

    exp_obj_equal = kwargs['exp_obj_equal']

    assert eq1 == exp_obj_equal
    assert eq2 == exp_obj_equal
    assert ne1 != exp_obj_equal
    assert ne2 != exp_obj_equal


@pytest.mark.parametrize(
    "desc, kwargs, exp_exc_types, exp_warn_types, condition",
    TESTCASES_NOCASEDICT_EQUAL_HASH + TESTCASES_NOCASEDICT_HASH)
@pytest_extensions.test_function
def test_NocaseDict_hash(
        desc, kwargs, exp_exc_types, exp_warn_types, condition):
    """
    All test cases for NocaseDict.__hash__().
    """

    obj1 = kwargs['obj1']
    obj2 = kwargs['obj2']

    # Double check they are different objects
    assert id(obj1) != id(obj2)

    # The code to be tested
    hash1 = hash(obj1)
    hash2 = hash(obj2)

    exp_hash_equal = kwargs['exp_obj_equal']

    assert (hash1 == hash2) == exp_hash_equal


class TestOrdering(BaseTest):
    """Verify that ordering comparisons between NocaseDict instances
    issue a deprecation warning, and for Python 3, in addition raise
    TypeError."""

    def assertWarning(self, comp_str):
        """Common function for assert warning"""
        with warnings.catch_warnings(record=True) as wlist:
            warnings.simplefilter("always")
            if six.PY2:
                eval(comp_str)  # pylint: disable=eval-used
            else:
                try:
                    eval(comp_str)  # pylint: disable=eval-used
                except TypeError as exc:
                    msg = str(exc)
                    if "not supported between instances" not in msg and \
                            "unorderable types" not in msg:
                        self.fail("Applying ordering to a dictionary in "
                                  "Python 3 did raise TypeError but with an "
                                  "unexpected message: %s" % msg)
                except Exception as exc:  # pylint: disable=broad-except
                    msg = str(exc)
                    self.fail("Applying ordering to a dictionary in Python 3 "
                              "did not raise TypeError, but %s: %s" %
                              (exc.__class__.__name__, msg))
                else:
                    self.fail("Applying ordering to a dictionary in Python 3 "
                              "succeeded (should not happen)")
            assert len(wlist) >= 1
            assert issubclass(wlist[-1].category, DeprecationWarning)
            assert "deprecated" in str(wlist[-1].message)

    def test_all(self):
        """Test for the compare options that should generate assertWarning"""
        self.assertWarning("self.dic < self.dic")
        self.assertWarning("self.dic <= self.dic")
        self.assertWarning("self.dic > self.dic")
        self.assertWarning("self.dic >= self.dic")


class TestContains(BaseTest):
    """Class for dict contains functionality"""
    def test_all(self):
        """Method for test dict contains functionality"""
        self.assertTrue('dog' in self.dic)
        self.assertTrue('Dog' in self.dic)
        self.assertTrue('Cat' not in self.dic)


class TestForLoop(BaseTest):
    """Class for test for loop with dictionary"""

    def test_all(self):
        """Test method for TestForLoop"""
        keys = set()
        for key in self.dic:
            keys.add(key)
        self.assertEqual(keys, set(['Budgie', 'Dog']))

    def test_order_preservation(self):
        """Test order preservation of for loop on dict"""
        dic = NocaseDict()
        for key, value in self.order_tuples:
            dic[key] = value
        i = 0
        for key in dic:
            item = (key, dic[key])
            exp_item = self.order_tuples[i]
            self.assertEqual(item, exp_item)
            i += 1


class TestIterkeys(BaseTest):
    """Class for iterkeys test"""

    def test_all(self):
        """iterkeys test method"""
        keys = set()
        for key in self.dic.iterkeys():
            keys.add(key)
        self.assertEqual(keys, set(['Budgie', 'Dog']))

    def test_order_preservation(self):
        """Test order preservation of iterkeys()"""
        dic = NocaseDict()
        for key, value in self.order_tuples:
            dic[key] = value
        i = 0
        for key in dic.iterkeys():
            item = (key, dic[key])
            exp_item = self.order_tuples[i]
            self.assertEqual(item, exp_item)
            i += 1


class TestItervalues(BaseTest):
    """Class for test itervalues test"""
    def test_all(self):
        """itervalues test method"""
        vals = set()
        for val in self.dic.itervalues():
            vals.add(val)
        self.assertEqual(vals, set(['Cat', 'Fish']))

    def test_order_preservation(self):
        """Test order preservation of itervalues()"""
        dic = NocaseDict()
        for key, value in self.order_tuples:
            dic[key] = value
        i = 0
        for value in dic.itervalues():
            exp_value = self.order_tuples[i][1]
            self.assertEqual(value, exp_value)
            i += 1


class TestIteritems(BaseTest):
    """Class to test iteritems for dict"""

    def test_all(self):
        """Method for test iteritems for dict"""
        items = set()
        for item in self.dic.iteritems():
            items.add(item)
        self.assertEqual(items, set([('Budgie', 'Fish'), ('Dog', 'Cat')]))

    def test_order_preservation(self):
        """Test order preservation of iteritems()"""
        dic = NocaseDict()
        for key, value in self.order_tuples:
            dic[key] = value
        i = 0
        for item in dic.iteritems():
            exp_item = self.order_tuples[i]
            self.assertEqual(item, exp_item)
            i += 1


class TestRepr(unittest.TestCase):
    """Class to test repr functionality for NocaseDict"""

    def test_reliable_order(self):
        """Test that repr() has a reliable result for two dicts with the same
        insertion order."""

        dic1 = NocaseDict()
        dic1['Budgie'] = 'Fish'
        dic1['Foo'] = 'Bla'
        dic1['Dog'] = 'Cat'

        dic2 = NocaseDict()
        dic2['Budgie'] = 'Fish'
        dic2['Foo'] = 'Bla'
        dic2['Dog'] = 'Cat'

        self.assertEqual(repr(dic1), repr(dic2))


class Test_unnamed_keys(object):
    """Class to test unnamed keys (key is None) for NocaseDict"""

    def test_unnamed_keys(self):
        """Test unnamed keys."""

        dic = NocaseDict()
        dic.allow_unnamed_keys = True

        dic[None] = 'a'
        assert None in dic
        assert len(dic) == 1

        a_val = dic[None]
        assert a_val == 'a'

        del dic[None]
        assert None not in dic
        assert not dic


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

# Copyright 2017 InovaDevelopment Inc.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

"""
Unit test logging functionality in _logging.py
"""

from __future__ import absolute_import, print_function

import os
import re
import logging

# Allows use of lots of single character variable names.
# pylint: disable=invalid-name,missing-docstring,too-many-statements
# pylint: disable=too-many-lines,no-self-use
import unittest
import pytest

# logging_tree is a useful tool to understand what the logging
# configuraiton produces.  It is not normally installed in pywbem but simply
# documented here for anyone changing these tests. You must manually install
# it with pip. It displays a tree of loggers with pertinent information about
# each logger.
# from logging_tree import printout
# To use the tool just set a line to printout() where you want to see what
# loggers exist

from testfixtures import LogCapture, log_capture, compare, TempDirectory
from pywbem import WBEMConnection
from pywbem._logging import configure_loggers_from_string, configure_logger, \
    LOGGER_API_CALLS_NAME, LOGGER_HTTP_NAME

VERBOSE = False

# Location of any test scripts for testing wbemcli.py
SCRIPT_DIR = os.path.dirname(__file__)

LOG_FILE_NAME = 'test_logging.log'
TEST_OUTPUT_LOG = '%s/%s' % (SCRIPT_DIR, LOG_FILE_NAME)


@pytest.fixture(autouse=True)
def tmp_dir():
    with TempDirectory() as dir:
        yield dir


class BaseLoggingTest(unittest.TestCase):
    """
    Methods required by all tests
    """


class UnitLoggingTests(BaseLoggingTest):
    """Base class for logging unit tests"""

    def setUp(self):
        """Setup that is run before each test method."""
        WBEMConnection._reset_logging_config()

    def logger_validate(self, log_name, log_dest, detail_level,
                        log_filename=None):
        """
        Test for correct definition of one logger. Creates the logger and
        tests the logger parameters against expected results. This does not
        create any logs.
        """
        if log_name == 'all':
            self.logger_validate('api', log_dest, detail_level=detail_level,
                                 log_filename=log_filename)
            self.logger_validate('http', log_dest, detail_level,
                                 log_filename=log_filename)

        else:
            if log_name == 'api':
                logger = logging.getLogger(LOGGER_API_CALLS_NAME)
            elif log_name == 'http':
                logger = logging.getLogger(LOGGER_HTTP_NAME)
            else:
                self.fail('Input error. log_name %s ' % log_name)

            # TODO compare detail_level

            compare(logger.level, 10)
            if logger.handlers[0]:
                self.assertEqual(len(logger.handlers), 1)
                if log_dest == 'file':
                    self.assertTrue(isinstance(logger.handlers[0],
                                               logging.FileHandler))
                    self.assertTrue(log_filename is not None)
                elif log_dest == 'stderr':
                    self.assertTrue(isinstance(logger.handlers[0],
                                               logging.StreamHandler))
            else:
                self.fail('No logger defined')

    def configure_logger_test(self, log_name, log_dest=None, detail_level=None,
                              log_filename=None, error=None):
        """
        Unified test function for the configure_logger function
        """
        if error:
            try:
                configure_logger(log_name, log_dest=log_dest,
                                 detail_level=detail_level,
                                 log_filename=log_filename)
                self.fail('Exception expected')
            except ValueError:
                pass
        else:
            configure_logger(log_name, log_dest=log_dest,
                             detail_level=detail_level,
                             log_filename=log_filename)

            self.logger_validate(log_name, log_dest, detail_level,
                                 log_filename=log_filename)

    def loggers_from_string_test(self, param, expected_result, log_file=None,
                                 connection_defined=False):
        """ Common test for successful parsing"""
        # pylint: disable=protected-access

        # logging handlers are static.  We must clear them between tests
        # Remove any handlers from loggers for this test
        api_logger = logging.getLogger(LOGGER_API_CALLS_NAME)
        api_logger.handlers = []
        http_logger = logging.getLogger(LOGGER_HTTP_NAME)
        http_logger.handlers = []

        if connection_defined:
            self.fail('TODO: Test with connections not yet implemented')
            conn = WBEMConnection('http:/blah')
        else:
            conn = True  # for all future connections

        if expected_result == 'error':
            try:
                configure_loggers_from_string(param, log_filename=log_file,
                                              connection=conn, propagate=True)
                self.fail('Exception expected')
            except ValueError:
                pass
        else:
            configure_loggers_from_string(param, log_filename=log_file,
                                          connection=conn, propagate=True)

            api_logger = logging.getLogger(LOGGER_API_CALLS_NAME)
            http_logger = logging.getLogger(LOGGER_HTTP_NAME)

            if 'level' in expected_result:
                level = expected_result['level']
                if level[0]:
                    compare(api_logger.level, level[0])
                if level[1]:
                    compare(http_logger.level, level[1])

            if 'handler' in expected_result:
                handler = expected_result['handler']
                if handler[0]:
                    self.assertEqual(len(api_logger.handlers), 1)
                    self.assertTrue(isinstance(api_logger.handlers[0],
                                               handler[0]))
                if handler[1]:
                    self.assertEqual(len(http_logger.handlers), 1)
                    self.assertTrue(isinstance(http_logger.handlers[0],
                                               handler[1]))

            if 'detail' in expected_result:
                exp_detail_levels = expected_result['detail']
                if connection_defined:
                    # TODO add test for when connection param exists
                    # need to get to recorder and test detail level
                    self.fail('TODO: Test with connections not yet implemented')
                else:
                    detail_levels = WBEMConnection._log_detail_levels
                    self.assertTrue(detail_levels['api'], exp_detail_levels[0])
                    self.assertTrue(detail_levels['http'], exp_detail_levels[1])

        # remove handlers from our loggers.
        for h in api_logger.handlers:
            api_logger.removeHandler(h)
            h.flush()
            h.close()
        for h in http_logger.handlers:
            http_logger.removeHandler(h)
            h.flush()
            h.close()

        # Close log file
        if log_file:
            if os.path.exists(log_file):
                os.remove(log_file)


class TestLoggersFromString(UnitLoggingTests):
    """
    Test the configure_loggers_from_string and configure_logger
    functions. Some of the logging configuration methods are in WBEMConnection
    """

    def test_comp_only(self):
        """'Test all' string"""
        param = 'all'
        self.loggers_from_string_test(param, {'level': (10, 10),
                                              'handler': (logging.FileHandler,
                                                          logging.FileHandler),
                                              'detail': ('all', 'all')},
                                      log_file='blah.log')

    def test_comp_only2(self):
        """'Test all=' string"""
        param = 'all='
        self.loggers_from_string_test(param, {'level': (10, 10),
                                              'handler': (logging.FileHandler,
                                                          logging.FileHandler),
                                              'detail': ('all', 'all')},
                                      log_file='blah.log')

    def test_complete1(self):
        """Test all=file string"""
        param = 'all=file'
        self.loggers_from_string_test(param, {'level': (10, 10),
                                              'handler': (logging.FileHandler,
                                                          logging.FileHandler),
                                              'detail': ('all', 'all')},
                                      log_file='blah.log')

    def test_complete2(self):
        """Test all=file:summary string"""
        param = 'all=file:summary'
        self.loggers_from_string_test(param, {'level': (10, 10),
                                              'handler': (logging.FileHandler,
                                                          logging.FileHandler),
                                              'detail': ('summary', 'summary')},

                                      log_file='blah.log')

    def test_complete3(self):
        """Test all=file:summary string"""
        param = 'all=file:10'
        self.loggers_from_string_test(param, {'level': (10, 10),
                                              'handler': (logging.FileHandler,
                                                          logging.FileHandler),
                                              'detail': (10, 10)},

                                      log_file='blah.log')

    def test_multiple1(self):
        """Test api=file:min,http=file:all string"""
        param = 'api=file,http=file:all'
        self.loggers_from_string_test(param, {'level': (10, 10),
                                              'handler': (logging.FileHandler,
                                                          logging.FileHandler)},
                                      log_file='blah.log')

    def test_multiple2(self):
        """Test api=file:min,http=file:min string"""
        param = 'api=file,http=stderr'
        self.loggers_from_string_test(param, {'level': (10, 10),
                                              'handler':
                                              (logging.FileHandler,
                                               logging.StreamHandler)},
                                      log_file='blah.log')

    def test_invalid_logname(self):
        """Test for exception, log name invalid.  'blah' """
        param = 'blah'
        self.loggers_from_string_test(param, 'error')

    def test_to_many_params(self):
        """ test all=file:junk string """
        param = "all=file:all:junk"
        self.loggers_from_string_test(param, 'error')

    def test_empty(self):
        param = ""
        self.loggers_from_string_test(param, 'error')


class TestDefineLogger(UnitLoggingTests):
    """ Test the configure_logger method."""

    def test_configue_single_logger1(self):
        """
        Create a simple logger
        """
        self.configure_logger_test('api', 'file', detail_level='all',
                                   log_filename=TEST_OUTPUT_LOG)

    def test_create_single_logger2(self):
        """
        Create a simple logger from detailed parameter input
        """
        self.configure_logger_test('http', 'file', detail_level='all',
                                   log_filename=TEST_OUTPUT_LOG)

    def test_create_single_logger3(self):
        """
        Create a simple logger from detailed parameter input
        """
        self.configure_logger_test('http', 'stderr', detail_level='all',
                                   log_filename=TEST_OUTPUT_LOG)

    def test_create_single_logger4(self):
        """
        Create a simple logger from detailed parameter input
        """
        self.configure_logger_test('all', 'stderr', detail_level='all',
                                   log_filename=TEST_OUTPUT_LOG)

    def test_create_single_logger5(self):
        """
        Create a simple logger from detailed parameter input
        """
        self.configure_logger_test('all', 'stderr', detail_level='all',
                                   log_filename=TEST_OUTPUT_LOG)

    def test_create_single_loggerEr1(self):
        """
        Create a simple logger from detailed parameter input
        """
        self.configure_logger_test('api', 'blah', detail_level='all',
                                   error=True)
        self.configure_logger_test('http', 'blah', detail_level='all',
                                   error=True)
        self.configure_logger_test('api', 'blah', detail_level='al',
                                   error=True)
        self.configure_logger_test('http', 'blah', detail_level='al',
                                   error=True)


class BaseLoggingExecutionTests(BaseLoggingTest):
    """Base class for logging unit tests"""

    def setUp(self):
        """Setup that is run before each test method."""
        WBEMConnection._reset_logging_config()

    def tearDown(self):
        LogCapture.uninstall_all()
        logging.shutdown()
        if os.path.isfile(TEST_OUTPUT_LOG):
            os.remove(TEST_OUTPUT_LOG)


class TestLoggerOutput(BaseLoggingExecutionTests):
    """Test output from logging"""

    @log_capture()
    def test_log_output(self, lc):
        test_input = 'all=file'

        configure_loggers_from_string(
            test_input, TEST_OUTPUT_LOG, propagate=True)

        my_logger = logging.getLogger(LOGGER_API_CALLS_NAME)

        self.assertNotEqual(my_logger, None,
                            'Valid named logger %s expected.'
                            % LOGGER_API_CALLS_NAME)

        max_size = 1000
        result = 'This is fake return data'
        return_name = 'Return'
        if max_size and len(repr(result)) > max_size:
            result = '{:.{sz}s}...' \
                .format(repr(result), sz=max_size)
        else:
            result = '%s' % repr(result)

        my_logger.debug('%s: %s: %s', return_name, 'FakeMethodName', result)

        lc.check(('pywbem.api', 'DEBUG',
                 "Return: FakeMethodName: 'This is fake return data'"))


class TestLoggerPropagate(object):
    """Test logging with propagate parameter variations"""

    @pytest.mark.parametrize(
        "propagate", (True, False)
    )
    @pytest.mark.parametrize(
        "logger_names", (
            ('api', 'pywbem.api'),
            # http logger does not log WBEMConnection
        )
    )
    def test_logger_propagate(self, tmp_dir, logger_names, propagate):

        short_name, logger_name = logger_names

        # The testing approach is to log to files and check their contents.
        # Neither LogCapture nor OutputCapture seemed to work with pytest.
        logger_filename = os.path.join(tmp_dir.path, 'pywbem.xxx.log')
        pkg_filename = os.path.join(tmp_dir.path, 'pywbem.log')

        # Create a log handler on the 'pywbem.<xxx>' logger to be tested
        configure_logger(short_name, log_dest='file',
                         log_filename=logger_filename,
                         detail_level='all', connection=True,
                         propagate=propagate)

        # Create a log handler on the 'pywbem' logger (parent)
        pkg_logger = logging.getLogger('pywbem')
        pkg_handler = logging.FileHandler(pkg_filename, encoding="UTF-8")
        pkg_handler.setLevel(logging.DEBUG)
        pkg_formatter = logging.Formatter('%(asctime)s-%(name)s-%(message)s')
        pkg_handler.setFormatter(pkg_formatter)
        pkg_logger.addHandler(pkg_handler)

        # Create a log event
        WBEMConnection('bla')

        # Verify the 'propagate' attribute of the logger to be tested
        logger = logging.getLogger(logger_name)
        assert logger.propagate == propagate

        for h in logger.handlers + pkg_logger.handlers:
            try:
                h.flush()
                h.close()
            except AttributeError:
                pass

        pkg_logger.removeHandler(pkg_handler)

        with open(logger_filename) as logger_fp:
            logger_line = logger_fp.read()
        assert re.match(r'.*-%s\..*-Connection:' % logger_name, logger_line)

        with open(pkg_filename) as pkg_fp:
            pkg_line = pkg_fp.read()
        if propagate:
            assert re.match(r'.*-pywbem.*-Connection:', pkg_line)
        else:
            assert pkg_line == ''


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
"""
    Test to compare results of DOM tupletree implementation with the
    sax implementation
"""
import os
import xml
import pprint
import unittest
import six
import pytest
from pkg_resources import resource_filename

from pywbem import tupletree, ParseError

pp = pprint.PrettyPrinter(indent=4)  # pylint: disable=invalid-name

test_tuple = (u'CIM',  # pylint: disable=invalid-name
              {u'CIMVERSION': u'2.0', u'DTDVERSION': u'2.0'},
              [(u'MESSAGE',
                {u'ID': u'1001', u'PROTOCOLVERSION': u'1.0'},
                [u'\n',
                 (u'SIMPLERSP',
                  {},
                  [(u'IMETHODRESPONSE',
                    {u'NAME': u'Associators'},
                    [(u'IRETURNVALUE',
                      {},
                      [u'\n    \n',
                       (u'VALUE.OBJECTWITHPATH',
                        {},
                        [u'\n',
                         (u'INSTANCEPATH',
                          {},
                          [u'\n',
                           (u'NAMESPACEPATH',
                            {},
                            [u'\n',
                             (u'HOST',
                              {},
                              [u'10.10.10.10'],
                              None),
                             u'\n',
                             (u'LOCALNAMESPACEPATH',
                              {},
                              [u'\n',
                               (u'NAMESPACE',
                                {u'NAME': u'root'},
                                [],
                                None),
                               u'\n',
                               (u'NAMESPACE',
                                {u'NAME': u'emc'},
                                [],
                                None),
                               u'\n'],
                              None),
                             u'\n'],
                            None),
                           u'\n',
                           (u'INSTANCENAME',
                            {u'CLASSNAME': u'Symm_StorageVolume'},
                            [u'\n',
                             (u'KEYBINDING',
                              {u'NAME': u'CreationClassName'},
                              [(u'KEYVALUE',
                                {u'TYPE': u'string',
                                 u'VALUETYPE': u'string'},
                                [u'Symm_StorageVolume'],
                                None)],
                              None),
                             u'\n',
                             (u'KEYBINDING',
                              {u'NAME': u'DeviceID'},
                              [(u'KEYVALUE',
                                {u'TYPE': u'string',
                                 u'VALUETYPE': u'string'},
                                [u'00000'],
                                None)],
                              None),
                             u'\n',
                             (u'KEYBINDING',
                              {u'NAME': u'SystemCreationClassName'},
                              [(u'KEYVALUE',
                                {u'TYPE': u'string',
                                 u'VALUETYPE': u'string'},
                                [u'Symm_StorageSystem'],
                                None)],
                              None),
                             u'\n',
                             (u'KEYBINDING',
                              {u'NAME': u'SystemName'},
                              [(u'KEYVALUE',
                                {u'TYPE': u'string',
                                 u'VALUETYPE': u'string'},
                                [u'SYMMETRIX-+-000194900000'],
                                None)],
                              None),
                             u'\n'],
                            None),
                           u'\n'],
                          None),
                         u'\n',
                         (u'INSTANCE',
                          {u'CLASSNAME': u'Symm_StorageVolume'},
                          [u'\n',
                           (u'PROPERTY',
                            {u'NAME': u'Usage',
                             u'TYPE': u'uint16'},
                            [(u'VALUE',
                              {},
                              [u'3'],
                              None),
                             u'\n'],
                            None),
                           u'\n'],
                          None),
                         u'\n'],
                        None),
                       u'\n\n'],
                      None)],
                    None)],
                  None)],
                None)],
              None)


# The following was originally the pywbem parser.  Now it is uses simply
# as a check on the sax parser.  Therefore, this code was moved from
# tupletree.py to here for the tests. Nov 2016. Note that this code is
# referenced from other code in the attic so we might not want to eliminate
# in completely.
def dom_to_tupletree(node):
    """Convert a DOM object to a pyRXP-style tuple tree.

    Each element is a 4-tuple of (NAME, ATTRS, CONTENTS, None).

    Very nice for processing complex nested trees.
    """

    if node.nodeType == node.DOCUMENT_NODE:
        # boring; pop down one level
        return dom_to_tupletree(node.firstChild)
    assert node.nodeType == node.ELEMENT_NODE

    name = node.nodeName
    attrs = {}
    contents = []

    for child in node.childNodes:
        if child.nodeType == child.ELEMENT_NODE:
            contents.append(dom_to_tupletree(child))
        elif child.nodeType == child.TEXT_NODE:
            assert isinstance(child.nodeValue, six.string_types), \
                "text node %s is not a string %r" % child
            contents.append(child.nodeValue)
        elif child.nodeType == child.CDATA_SECTION_NODE:
            contents.append(child.nodeValue)
        else:
            raise RuntimeError("can't handle %r" % child)

    for i in range(node.attributes.length):
        attr_node = node.attributes.item(i)
        attrs[attr_node.nodeName] = attr_node.nodeValue

    # TODO: Cannot handle comments, cdata, processing instructions, etc.

    # it's so easy in retrospect!
    return (name, attrs, contents, None)


def xml_to_tupletree(xml_string):
    """
    Parse XML straight into tupletree.
    Uses the minidom to parse xml_string int a dom object.
    This is part of the old obsolete dom code
    """
    dom_xml = xml.dom.minidom.parseString(xml_string)
    return dom_to_tupletree(dom_xml)


class TestTupleTreeRegression(unittest.TestCase):
    """Setup the TupleTree tests"""
    def setUp(self):
        self.data_dir = resource_filename(__name__, 'tupletree_ok')
        # pylint: disable=unused-variable
        for path, dirs, files in os.walk(self.data_dir):
            self.filenames = [os.path.join(path, f) for f in files]

    def test_to_tupletree(self):
        """
        SAX parsing replicates DOM parsing to the tupletree.

        This test compares the input xml compiled with the dom and sax.

        """
        for fn in self.filenames:
            with open(fn, 'rb') as fh:
                xml_str = fh.read()
            tree_dom = xml_to_tupletree(xml_str)
            tree_sax = tupletree.xml_to_tupletree_sax(xml_str, 'Test XML')
            self.assertEqual(tree_dom, tree_sax,
                             'SAX and DOM not equal for %s:\n%s,\n%s'
                             % (fn, pp.pformat(tree_sax), pp.pformat(tree_dom)))


class TestTupleTreeError(unittest.TestCase):
    """Base class for Testing Tuple tree errors. Does setUp and common
       method to test
    """
    def setUp(self):
        """Unittest setUp"""
        self.data_dir = resource_filename(__name__, 'tupletree_error')

    def test_to_tupletree_error(self):
        """SAX parsing generates errors."""
        filename = os.path.join(self.data_dir, 'Associators_error.xml')
        with open(filename, 'rb') as fh:
            xml_str = fh.read()
            self.assertRaises(ParseError,
                              tupletree.xml_to_tupletree_sax,
                              xml_str, 'Test XML')


class TestTupleTreeSax(unittest.TestCase):
    """Class to execute SaxTupleTree tests"""
    def test_xml_to_tupletree_sax(self):
        """
        XML to tupletree with SAX is accurate.

        This test compares data to the test_tuple variable in this file.
        """
        data_dir = resource_filename(__name__, 'tupletree_ok')
        path = os.path.join(data_dir, 'Associators_StorageVolume_small.xml')
        with open(path, 'rb') as fh:
            xml_str = fh.read()
        tree_sax = tupletree.xml_to_tupletree_sax(xml_str, 'Test XML')
        self.assertEqual(test_tuple, tree_sax,
                         'Test tuple and SAX not equal for %s:\n%s,\n%s'
                         % (path, pp.pformat(tree_sax), pp.pformat(tree_sax)))


class Test_check_invalid_utf8_sequences(object):
    """Tests for check_invalid_utf8_sequences()"""

    testcases = [
        # Each testcase has these items:
        # * desc: Short testcase description.
        # * utf8_string: Input string for the function.
        # * exp_exc_type: Expected exception type, or None if expecting success.
        # * condition: Condition for testcase to run.

        # General cases
        (
            "Incorrect type of input string",
            u'<V>ab</V>',
            TypeError, True
        ),

        # Good cases
        (
            "Good case without special chars",
            b'<V>a</V>',
            None, True
        ),
        (
            "Good case with U+0009 (TAB), U+000A (NL), U+000D (CR) chars",
            b'<V>a\x09b\x0Ac\x0Dd</V>',
            None, True
        ),
        (
            "Good case with U+0350 char (in UTF-8)",
            b'<V>a\xCD\x90b</V>',
            None, True
        ),
        (
            "Good case with U+2013 char (in UTF-8)",
            b'<V>a\xE2\x80\x93b</V>',
            None, True
        ),
        (
            "Good case with U+10122 char (in UTF-8)",
            b'<V>a\xF0\x90\x84\xA2b</V>',
            None, True
        ),

        # Correctly encoded but ill-formed UTF-8
        (
            "Ill-formed UTF-8 using surrogate U+D800,U+DD22 "
            "(in otherwise correctly encoded UTF-8)",
            b'<V>a\xED\xA0\x80\xED\xB4\xA2b</V>',
            ParseError, True
        ),
        (
            "Ill-formed UTF-8 using surrogates U+D800,U+DD22 and U+D800,U+DD23 "
            "(in otherwise correctly encoded UTF-8)",
            b'<V>a\xED\xA0\x80\xED\xB4\xA2b\xED\xA0\x80\xED\xB4\xA3</V>',
            ParseError, True
        ),

        # Incorrectly encoded UTF-8
        (
            "Incorrectly encoded UTF-8 with 1-byte sequence",
            b'<V>a\x80b</V>',
            ParseError, True
        ),
        (
            "Incorrectly encoded UTF-8 with 2-byte sequence "
            "with missing second byte",
            b'<V>a\xC0',
            ParseError, True
        ),
        (
            "Incorrectly encoded UTF-8 with 2-byte sequence "
            "with incorrect 2nd byte",
            b'<V>a\xC0b</V>',
            ParseError, True
        ),
        (
            "Incorrectly encoded UTF-8 with 4-byte sequence "
            "with incorrect 3rd byte",
            b'<V>a\xF1\x80abc</V>',
            ParseError, True
        ),
        (
            "Incorrectly encoded UTF-8 with 4-byte sequence "
            "with incorrect 3rd byte that is an incorrect new start",
            b'<V>a\xF1\x80\xFFbc</V>',
            ParseError, True
        ),
        (
            "Incorrectly encoded UTF-8 with 4-byte sequence "
            "with incorrect 3rd byte that is a correct new start",
            b'<V>a\xF1\x80\xC2\x81c</V>',
            ParseError, True
        ),
    ]

    @pytest.mark.parametrize(
        "desc, utf8_string, exp_exc_type, condition",
        testcases
    )
    def test_check_invalid_utf8_sequences(
            self, desc, utf8_string, exp_exc_type, condition):
        """Tests for check_invalid_utf8_sequences()"""

        if not condition:
            pytest.skip("Condition for test case not met")

        if exp_exc_type is None:

            unicode_string = tupletree.check_invalid_utf8_sequences(
                utf8_string, desc)

            assert isinstance(unicode_string, six.text_type), desc
            utf8_string_u = utf8_string.decode('utf-8')
            assert unicode_string == utf8_string_u, desc

        else:
            with pytest.raises(exp_exc_type):

                tupletree.check_invalid_utf8_sequences(utf8_string, desc)


class Test_check_invalid_xml_chars(object):
    """Tests for check_invalid_xml_chars()"""

    testcases = [
        # Each testcase has these items:
        # * desc: Short testcase description.
        # * xml_string: Input string for the function.
        # * exp_exc_type: Expected exception type, or None if expecting success.
        # * condition: Condition for testcase to run.

        # General cases
        (
            "Incorrect type of input string",
            b'<V>ab</V>',
            TypeError, True
        ),

        # Good cases
        (
            "Good case without special chars",
            u'<V>a</V>',
            None, True
        ),
        (
            "Good case with U+0009 (TAB), U+000A (NL), U+000D (CR) chars",
            u'<V>a\u0009b\u000Ac\u000Dd</V>',
            None, True
        ),
        (
            "Good case with U+0350 char",
            u'<V>a\u0350b</V>',
            None, True
        ),
        (
            "Good case with U+2013 char",
            u'<V>a\u2013b</V>',
            None, True
        ),
        (
            "Good case with U+10122 char",
            u'<V>a\u010122b</V>',
            None, True
        ),

        # Invalid XML characters
        (
            "Invalid XML char U+0008 (BEL)",
            u'<V>a\u0008b</V>',
            ParseError, True
        ),
        (
            "Invalid XML char U+0000",
            u'<V>a\u0000b</V>',
            ParseError, True
        ),
        (
            "Invalid XML chars U+0001 ... U+0008",
            u'<V>a\u0001b\u0002c\u0003d\u0004e\u0005f\u0006g\u0007h\u0008i</V>',
            ParseError, True
        ),
        (
            "Invalid XML chars U+000B ... U+000C",
            u'<V>a\u000Bb\u000Cc</V>',
            ParseError, True
        ),
        (
            "Invalid XML chars U+000E ... U+0015",
            u'<V>a\u000Eb\u000Fc\u0010d\u0011e\u0012f\u0013g\u0014h\u0015i</V>',
            ParseError, True
        ),
        (
            "Invalid XML chars U+0016 ... U+001D",
            u'<V>a\u0016b\u0017c\u0018d\u0019e\u001Af\u001Bg\u001Ch\u001Di</V>',
            ParseError, True
        ),
        (
            "Invalid XML chars U+001E ... U+001F",
            u'<V>a\u001Eb\u001Fc</V>',
            ParseError, True
        ),
    ]

    @pytest.mark.parametrize(
        "desc, xml_string, exp_exc_type, condition",
        testcases
    )
    def test_check_invalid_xml_chars(
            self, desc, xml_string, exp_exc_type, condition):
        """Tests for check_invalid_xml_chars()"""

        if not condition:
            pytest.skip("Condition for test case not met")

        if exp_exc_type is None:

            tupletree.check_invalid_xml_chars(xml_string, desc)

        else:
            with pytest.raises(exp_exc_type):

                tupletree.check_invalid_xml_chars(xml_string, desc)


if __name__ == '__main__':
    unittest.main()



#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

"""
The :class:`~pywbem.ValueMapping` class supports translating the values of an
integer-typed CIM element (e.g. property, method, or parameter) that is
qualified with the `ValueMap` and `Values` qualifiers, to the corresponding
value of the `Values` qualifier.

This class supports value ranges (e.g. ``"4..6"``) and the unclaimed marker
(``".."``).
"""

import re
import six

from .cim_types import CIMInt, type_from_name

__all__ = ['ValueMapping']


class ValueMapping(object):
    """
    *New in pywbem 0.9 as experimental and finalized in 0.10.*

    A utility class that supports translating the values of an integer-typed
    CIM element (property, method, parameter) that is qualified with the
    `ValueMap` and `Values` qualifiers, to the corresponding value of the
    `Values` qualifier.

    This is done by retrieving the CIM class definition defining the CIM
    element in question, and by inspecting its `ValueMap` and `Values`
    qualifiers.

    The translation of the values is performed by the
    :meth:`~pywbem.ValueMapping.tovalues` method.

    Instances of this class must be created through one of the factory class
    methods: :meth:`~pywbem.ValueMapping.for_property`,
    :meth:`~pywbem.ValueMapping.for_method`, or
    :meth:`~pywbem.ValueMapping.for_parameter`.

    Value ranges (``"2..4"``) and the indicator for unclaimed values (``".."``)
    in the `ValueMap` qualifier are supported.

    Example:

      Given the following definition of a property in MOF:

      .. code-block:: text

          class CIM_Foo {

                [ValueMap{ "0", "2..4", "..6", "7..", "9", ".." },
                 Values{ "zero", "two-four", "five-six", "seven-eight", "nine",
                 "unclaimed"}]
             uint16 MyProp;

          };

      Assuming this class exists in a WBEM server, the following code will
      create a value mapping for this property and will print a few property
      values and their corresponding `Values` strings::

          namespace = 'root/cimv2'
          conn = pywbem.WBEMConnection(...)  # WBEM server

          myprop_vm = pywbem.ValueMapping.for_property(
              conn, namespace, 'CIM_Foo', 'MyProp')

          print("value: Values")
          for value in range(0, 12):
              print("%5s: %r" % (value, myprop_vm.tovalues(value))

      Resulting output:

      .. code-block:: text

          value: Values
              0: 'zero'
              1: 'unclaimed'
              2: 'two-four'
              3: 'two-four'
              4: 'two-four'
              5: 'five-six'
              6: 'five-six'
              7: 'seven-eight'
              8: 'seven-eight'
              9: 'nine'
             10: 'unclaimed'
             11: 'unclaimed'
    """

    def __init__(self):

        self._conn = None
        self._namespace = None
        self._classname = None
        self._propname = None
        self._methodname = None
        self._parametername = None

        self._element_obj = None

        self._single_dict = {}  # for single values; elem_val: values_str)
        self._range_tuple_list = []  # for value ranges; tuple(lo,hi,values_str)
        self._unclaimed = None  # value of the unclaimed indicator '..'

    @classmethod
    def for_property(cls, server, namespace, classname, propname):
        # pylint: disable=line-too-long
        """
        Factory method that returns a new :class:`~pywbem.ValueMapping`
        instance that maps CIM property values to the `Values` qualifier
        defined on that property.

        If a `Values` qualifier is defined but no `ValueMap` qualifier, a
        default of 0-based consecutive numbers is applied (that is the default
        defined in :term:`DSP0004`).

        Parameters:

          server (:class:`~pywbem.WBEMConnection` or :class:`~pywbem.WBEMServer`):
            The connection to the WBEM server containing the namespace.

          namespace (:term:`string`):
            Name of the CIM namespace containing the class.
            If `None`, the default namespace of the connection will be used.

          classname (:term:`string`):
            Name of the CIM class exposing the property. The property can be
            defined in that class or inherited into that class.

          propname (:term:`string`):
            Name of the CIM property that defines the `Values` / `ValueMap`
            qualifiers.

        Returns:

            The new :class:`~pywbem.ValueMapping` instance.

        Raises:

            Exceptions raised by :class:`~pywbem.WBEMConnection`.
            ValueError: No `Values` qualifier defined.
            TypeError: The property is not integer-typed.
            KeyError: The property does not exist in the class.
        """  # noqa: E501

        conn = server
        try:
            get_class = conn.GetClass
        except AttributeError:
            conn = server.conn
            get_class = conn.GetClass

        class_obj = get_class(ClassName=classname,
                              namespace=namespace,
                              LocalOnly=False,
                              IncludeQualifiers=True)
        property_obj = class_obj.properties[propname]

        new_vm = cls._create_for_element(property_obj, conn, namespace,
                                         classname, propname=propname)

        return new_vm

    @classmethod
    def for_method(cls, server, namespace, classname, methodname):
        # pylint: disable=line-too-long
        """
        Factory method that returns a new :class:`~pywbem.ValueMapping`
        instance that maps CIM method return values to the `Values` qualifier
        of that method.

        If a `Values` qualifier is defined but no `ValueMap` qualifier, a
        default of 0-based consecutive numbers is applied (that is the default
        defined in :term:`DSP0004`).

        Parameters:

          server (:class:`~pywbem.WBEMConnection` or :class:`~pywbem.WBEMServer`):
            The connection to the WBEM server containing the namespace.

          namespace (:term:`string`):
            Name of the CIM namespace containing the class.

          classname (:term:`string`):
            Name of the CIM class exposing the method. The method can be
            defined in that class or inherited into that class.

          methodname (:term:`string`):
            Name of the CIM method that defines the `Values` / `ValueMap`
            qualifiers.

        Returns:

            The new :class:`~pywbem.ValueMapping` instance.

        Raises:

            Exceptions raised by :class:`~pywbem.WBEMConnection`.
            ValueError: No `Values` qualifier defined.
            TypeError: The method is not integer-typed.
            KeyError: The method does not exist in the class.
        """  # noqa: E501

        conn = server
        try:
            get_class = conn.GetClass
        except AttributeError:
            conn = server.conn
            get_class = conn.GetClass

        class_obj = get_class(ClassName=classname,
                              namespace=namespace,
                              LocalOnly=False,
                              IncludeQualifiers=True)
        method_obj = class_obj.methods[methodname]

        new_vm = cls._create_for_element(method_obj, conn, namespace,
                                         classname, methodname=methodname)

        return new_vm

    @classmethod
    def for_parameter(cls, server, namespace, classname, methodname,
                      parametername):
        # pylint: disable=line-too-long
        """
        Factory method that returns a new :class:`~pywbem.ValueMapping`
        instance that maps CIM parameter values to the `Values` qualifier
        defined on that parameter.

        If a `Values` qualifier is defined but no `ValueMap` qualifier, a
        default of 0-based consecutive numbers is applied (that is the default
        defined in :term:`DSP0004`).

        Parameters:

          server (:class:`~pywbem.WBEMConnection` or :class:`~pywbem.WBEMServer`):
            The connection to the WBEM server containing the namespace.

          namespace (:term:`string`):
            Name of the CIM namespace containing the class.

          classname (:term:`string`):
            Name of the CIM class exposing the method. The method can be
            defined in that class or inherited into that class.

          methodname (:term:`string`):
            Name of the CIM method that has the parameter.

          parametername (:term:`string`):
            Name of the CIM parameter that defines the `Values` / `ValueMap`
            qualifiers.

        Returns:

            The new :class:`~pywbem.ValueMapping` instance.

        Raises:

            Exceptions raised by :class:`~pywbem.WBEMConnection`.
            ValueError: No `Values` qualifier defined.
            TypeError: The parameter is not integer-typed.
            KeyError: The method does not exist in the class, or the
              parameter does not exist in the method.
        """  # noqa: E501

        conn = server
        try:
            get_class = conn.GetClass
        except AttributeError:
            conn = server.conn
            get_class = conn.GetClass

        class_obj = get_class(ClassName=classname,
                              namespace=namespace,
                              LocalOnly=False,
                              IncludeQualifiers=True)
        method_obj = class_obj.methods[methodname]
        parameter_obj = method_obj.parameters[parametername]

        new_vm = cls._create_for_element(parameter_obj, conn, namespace,
                                         classname, methodname=methodname,
                                         parametername=parametername)

        return new_vm

    @classmethod
    def _values_tuple(cls, i, valuemap_list, values_list, cimtype):
        """
        Return a tuple for the value range at position i, with these items:

        * lo - low value of the range
        * hi - high value of the range (can be equal to lo)
        * values - value of `Values` qualifier for this position

        Parameters:

          i (integer): position into valuemap_list and values_list

          valuemap_list (list of strings): `ValueMap` qualifier value

          values_list (list of strings): `Values` qualifier value

          cimtype (type): CIM type of the CIM element

        Raises:

            ValueError: Invalid `ValueMap` entry.
        """
        values_str = values_list[i]
        valuemap_str = valuemap_list[i]
        try:
            valuemap_int = int(valuemap_str)
            return (valuemap_int, valuemap_int, values_str)
        except ValueError:
            m = re.match(r'^([+-]?[0-9]*)\.\.([+-]?[0-9]*)$', valuemap_str)
            if m is None:
                raise ValueError("Invalid ValueMap entry: %r" % valuemap_str)
            lo = m.group(1)
            if lo == '':
                if i == 0:
                    lo = cimtype.minvalue
                else:
                    _, previous_hi, _ = cls._values_tuple(
                        i - 1, valuemap_list, values_list, cimtype)
                    lo = previous_hi + 1
            else:
                lo = int(lo)
            hi = m.group(2)
            if hi == '':
                if i == len(valuemap_list) - 1:
                    hi = cimtype.maxvalue
                else:
                    next_lo, _, _ = cls._values_tuple(
                        i + 1, valuemap_list, values_list, cimtype)
                    hi = next_lo - 1
            else:
                hi = int(hi)
            return (lo, hi, values_str)

    @classmethod
    def _create_for_element(cls, element_obj, conn, namespace, classname,
                            propname=None, methodname=None, parametername=None):
        # pylint: disable=line-too-long
        """
        Return a new :class:`~pywbem.ValueMapping` instance for the specified
        CIM element.

        If a `Values` qualifier is defined but no `ValueMap` qualifier, a
        default of 0-based consecutive numbers is applied (that is the default
        defined in :term:`DSP0004`).

        Parameters:

          element_obj (:class:`~pywbem.CIMProperty`, :class:`~pywbem.CIMMethod`, or :class:`~pywbem.CIMParameter`):
            The CIM element on which the qualifiers are defined.

          conn (:class:`~pywbem.WBEMConnection`):
            The connection to the WBEM server containing the namespace.

          namespace (:term:`string`):
            Name of the CIM namespace containing the class.

          classname (:term:`string`):
            Name of the CIM class exposing the method. The method can be
            defined in that class or inherited into that class.

          propname (:term:`string`):
            Name of the CIM property that defines the `Values` / `ValueMap`
            qualifiers.

          methodname (:term:`string`):
            Name of the CIM method that has the parameter.

          parametername (:term:`string`):
            Name of the CIM parameter that defines the `Values` / `ValueMap`
            qualifiers.

        Returns:

            The created :class:`~pywbem.ValueMapping` instance for the specified
            CIM element.

        Raises:

            ValueError: No `Values` qualifier defined.
            ValueError: Invalid `ValueMap` entry.
            TypeError: The CIM element is not integer-typed.
        """  # noqa: E501

        # pylint: disable=protected-access

        try:
            typename = element_obj.type  # Property, Parameter
        except AttributeError:
            typename = element_obj.return_type  # Method

        cimtype = type_from_name(typename)

        if not issubclass(cimtype, CIMInt):
            raise TypeError("The CIM element is not integer-typed: %s" %
                            typename)

        vm = ValueMapping()
        vm._element_obj = element_obj
        vm._conn = conn
        vm._namespace = namespace
        vm._classname = classname
        vm._propname = propname
        vm._methodname = methodname
        vm._parametername = parametername

        values_qual = element_obj.qualifiers.get('Values', None)
        if values_qual is None:
            # DSP0004 defines no default for a missing Values qualifier
            raise ValueError("No Values qualifier defined")
        values_list = values_qual.value

        valuemap_qual = element_obj.qualifiers.get('ValueMap', None)
        if valuemap_qual is None:
            # DSP0004 defines a default of consecutive index numbers
            vm._single_dict = dict(zip(range(0, len(values_list)), values_list))
            vm._range_tuple_list = []
            vm._unclaimed = None
        else:
            vm._single_dict = {}
            vm._range_tuple_list = []
            vm._unclaimed = None
            valuemap_list = valuemap_qual.value
            for i, valuemap_str in enumerate(valuemap_list):
                values_str = values_list[i]
                if valuemap_str == '..':
                    vm._unclaimed = values_str
                else:
                    lo, hi, values_str = cls._values_tuple(
                        i, valuemap_list, values_list, cimtype)
                    if lo == hi:
                        # single value
                        vm._single_dict[lo] = values_str
                    else:
                        # value range
                        vm._range_tuple_list.append((lo, hi, values_str))

        # pylint: enable=protected-access
        return vm

    def __repr__(self):
        """
        Return a representation of the :class:`~pywbem.ValueMapping` object
        with all attributes, that is suitable for debugging.
        """
        return "%s(_conn=%r, _namespace=%r, " \
               "_classname=%r, _propname=%r, _methodname=%r, " \
               "_parametername=%r, _element_obj=%r, " \
               "_single_dict=%r, _range_tuple_list=%r, _unclaimed=%r)" % \
               (self.__class__.__name__, self._conn, self._namespace,
                self._classname, self._propname, self._methodname,
                self._parametername, self._element_obj,
                self._single_dict, self._range_tuple_list, self._unclaimed)

    @property
    def conn(self):
        """
        :class:`~pywbem.WBEMConnection`:
        Connection to the WBEM server containing the CIM namespace (that
        contains the mapped CIM element).
        """
        return self._conn

    @property
    def namespace(self):
        """
        :term:`string`: Name of the CIM namespace containing the class that
        defines the mapped CIM element.
        """
        return self._namespace

    @property
    def classname(self):
        """
        :term:`string`: Name of the CIM class defining the mapped CIM element.
        """
        return self._classname

    @property
    def propname(self):
        """
        :term:`string`: Name of the CIM property that is mapped. `None`, if
        no property is mapped.
        """
        return self._propname

    @property
    def methodname(self):
        """
        :term:`string`: Name of the CIM method, that either is mapped itself,
        or that has the parameter that is mapped.  `None`, if no method or
        parameter is mapped.
        """
        return self._methodname

    @property
    def parametername(self):
        """
        :term:`string`: Name of the CIM parameter that is mapped. `None`, if
        no parameter is mapped.
        """
        return self._parametername

    @property
    def element(self):
        # pylint: disable=line-too-long
        """
        :class:`~pywbem.CIMProperty`, :class:`~pywbem.CIMMethod`, or :class:`~pywbem.CIMParameter`:
        The mapped CIM element.
        """  # noqa: E501
        return self._element_obj

    def tovalues(self, element_value):
        """
        Return the `Values` string for an element value, based upon this value
        mapping.

        Parameters:

          element_value (:term:`integer` or :class:`~pywbem.CIMInt`):
            The value of the CIM element (property, method, parameter).

        Returns:

          :term:`string`:
            The `Values` string for the element value.

        Raises:

          ValueError: Element value outside of the set defined by `ValueMap`.
          TypeError: Element value is not an integer type.
        """

        if not isinstance(element_value, (six.integer_types, CIMInt)):
            raise TypeError("Element value is not an integer type: %s" %
                            type(element_value))

        # try single value
        try:
            return self._single_dict[element_value]
        except KeyError:
            pass

        # try value ranges
        for range_tuple in self._range_tuple_list:
            lo, hi, values_str = range_tuple
            if lo <= element_value <= hi:
                return values_str

        # try catch-all '..'
        if self._unclaimed is not None:
            return self._unclaimed

        raise ValueError("Element value outside of the set defined by "
                         "ValueMap: %r" % element_value)



#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

"""
The following exceptions are pywbem specific exceptions that can be raised at
the WBEM client library API.
"""

from .cim_constants import _statuscode2name, _statuscode2string

# This module is meant to be safe for 'import *'.

__all__ = ['Error', 'ConnectionError', 'AuthError', 'HTTPError', 'TimeoutError',
           'VersionError', 'ParseError', 'CIMError']


class Error(Exception):
    """Base class for pywbem specific exceptions."""
    pass


class ConnectionError(Error):
    """This exception indicates a problem with the connection to the WBEM
    server. A retry may or may not succeed. Derived from
    :exc:`~pywbem.Error`."""
    pass


class AuthError(Error):
    """This exception indicates an authentication error with the WBEM server,
    either during TLS/SSL handshake, or during HTTP-level authentication.
    Derived from :exc:`~pywbem.Error`."""
    pass


class HTTPError(Error):
    """
    This exception indicates that the WBEM server returned an HTTP response
    with a bad HTTP status code. Derived from :exc:`~pywbem.Error`.

    The `args` instance variable is a `tuple (status, reason, cimerror,
    cimdetails)`.

    The `message` instance variable is not set.
    """

    # pylint: disable=super-init-not-called
    def __init__(self, status, reason, cimerror=None, cimdetails=None):
        """
        Parameters:

          status (:term:`integer`): HTTP status code (e.g. 500).

          reason (:term:`string`): HTTP reason phrase (e.g.
            "Internal Server Error").

          cimerror (:term:`string`): Value of the `CIMError` header field,
            if present. `None`, otherwise.

          cimdetails (dict): Dictionary with CIMOM-specific header
            fields with details about the situation reported in the `CIMError`
            header field.

            * Key: header field name (e.g. `PGErrorDetail`)
            * Value: header field value (i.e. text message)

            Passing `None` will result in an empty dictionary.
        """
        if cimdetails is None:
            cimdetails = {}
        self.args = (status, reason, cimerror, cimdetails)

    @property
    def status(self):
        """
        :term:`integer`: HTTP status code (e.g. 500).

        See :term:`RFC2616` for a list of HTTP status codes and reason phrases.
        """
        return self.args[0]

    @property
    def reason(self):
        """
        :term:`string`: HTTP reason phrase (e.g. "Internal Server Error").

        See :term:`RFC2616` for a list of HTTP status codes and reason phrases.
        """
        return self.args[1]

    @property
    def cimerror(self):
        """
        :term:`string`: Value of `CIMError` header field in response, if
        present. `None`, otherwise.

        See :term:`DSP0200` for a list of values.
        """
        return self.args[2]

    @property
    def cimdetails(self):
        """
        dict: CIMOM-specific details on the situation reported in the `CIMError`
        header field, with:

        * Key: header field name (e.g. `PGErrorDetail`).
        * Value: header field value.
        """
        return self.args[3]

    def __str__(self):
        ret_str = "%s (%s)" % (self.status, self.reason)
        if self.cimerror is not None:
            ret_str += ", CIMError: %s" % self.cimerror
        for key in self.cimdetails:
            ret_str += ", %s: %s" % (key, self.cimdetails[key])
        return ret_str


class TimeoutError(Error):
    """This exception indicates that the client timed out waiting for the WBEM
    server. Derived from :exc:`~pywbem.Error`."""
    pass


class ParseError(Error):
    """This exception indicates a parsing error with the CIM-XML response
    returned by the WBEM server, or in the CIM-XML request sent by the WBEM
    listener. Derived from :exc:`~pywbem.Error`."""
    pass


class VersionError(Error):
    """This exception indicates an unsupported CIM, DTD or protocol version
    with the CIM-XML response returned by the WBEM server, or in the CIM-XML
    request sent by the WBEM listener. Derived from :exc:`~pywbem.Error`."""
    pass


class CIMError(Error):
    """
    This exception indicates that the WBEM server returned an error response
    with a CIM status code. Derived from :exc:`~pywbem.Error`.

    Accessing the CIM status code of a :class:`CIMError` object:

      In Python 2, any :exc:`~py:exceptions.Exception` object can be accessed
      by index and slice and will delegate such access to its
      :attr:`~py:exceptions.BaseException.args` instance variable. In Python 3,
      that ability has been removed.

      In pywbem 0.9, the :attr:`~pywbem.CIMError.status_code` and
      :attr:`~pywbem.CIMError.status_description` properties were added.

      Therefore, the following approach is not recommended, because it does not
      work on Python 3::

          except CIMError as exc:
              status_code = exc[0]

      The following approach works for pywbem 0.7 or newer::

          except CIMError as exc:
              status_code = exc.args[0]

      The following approach is recommended when using pywbem 0.9 or newer::

          except CIMError as exc:
              status_code = exc.status_code
    """

    # pylint: disable=super-init-not-called
    def __init__(self, status_code, status_description=None):
        """
        Parameters:

          status_code (:term:`integer`): Numeric CIM status code.

          status_description (:term:`string`): CIM status description text
            returned by the server, representing a human readable message
            describing the error. `None`, if the server did not return
            a description text.

        :ivar args: A tuple (status_code, status_description) set from the
              corresponding init arguments.
        """
        self.args = (status_code, status_description)

    @property
    def status_code(self):
        """
        *New in pywbem 0.9.*

        :term:`integer`: Numeric CIM status code (e.g. 5).

        See :ref:`CIM status codes` for constants defining the numeric CIM
        status code values."""
        return self.args[0]

    @property
    def status_code_name(self):
        """
        *New in pywbem 0.9.*

        :term:`string`: Symbolic name of the CIM status code (e.g.
        "CIM_ERR_INVALID_CLASS").

        If the CIM status code is invalid, the string
        "Invalid status code <status_code>" is returned."""
        return _statuscode2name(self.status_code)

    @property
    def status_description(self):
        """
        *New in pywbem 0.9.*

        :term:`string`: CIM status description text returned by the server,
        representing a human readable message describing the error (e.g.
        "The specified class does not exist.").

        If the server did not return a description, a short default text for
        the CIM status code is returned. If the CIM status code is invalid,
        the string "Invalid status code <status_code>" is returned.
        """
        return self.args[1] or _statuscode2string(self.status_code)

    def __str__(self):
        return "%s: %s" % (self.status_code, self.status_description)



#!/usr/bin/env python
#

"""
Unittest functions in cim_operations.
"""

from __future__ import print_function, absolute_import

import os
import pytest

from pywbem import WBEMConnection, CIMError

from pywbem._recorder import LogOperationRecorder
from pywbem._recorder import TestClientRecorder as MyTestClientRecorder


class TestCreateConnection(object):
    """
    Test construction of WBEMConnection and those functions that do not
    depend on actually creating a connection
    """
    def test_connection_defaults(self):  # pylint: disable=no-self-use
        """Test creation of a connection with default constructor
           parameters.
        """
        conn = WBEMConnection('http://localhost')
        assert conn.url == 'http://localhost'
        assert conn.creds is None
        assert conn.x509 is None
        assert conn.use_pull_operations is False
        assert conn.stats_enabled is False

    @pytest.mark.parametrize(
        'attr_name, value', [
            ('url', 'http:/myserver'),
            ('creds', ('x', 'y')),
            ('default_namespace', 'foo'),
            ('x509', dict(cert_file='c', key_file='k')),
            ('verify_callback', lambda a, b, c, d, e: True),
            ('ca_certs', 'xxx'),
            ('no_verification', True),
            ('timeout', 30),
        ])
    def test_attrs_deprecated_setters(self, attr_name, value):
        """
        Test that the setting of certain attributes is deprecated.
        """
        conn = WBEMConnection('http://localhost')
        with pytest.warns(DeprecationWarning) as rec_warnings:

            # The code to be tested
            setattr(conn, attr_name, value)

        assert len(rec_warnings) == 1

        attr_value = getattr(conn, attr_name)
        assert attr_value == value

    @pytest.mark.parametrize(
        'attr_name, value', [
            ('last_raw_request', '<CIM/>'),
            ('last_raw_reply', '<CIM/>'),
            ('last_raw_reply', '<CIM/>'),
            ('last_request', '<CIM/>'),
            ('last_request_len', 7),
            ('last_reply_len', 7),
        ])
    def test_attrs_readonly(self, attr_name, value):
        """
        Test that certain attributes that were previously public are now
        read-only and that the original value has not changed when trying to
        modify them.
        """
        conn = WBEMConnection('http://localhost')
        value1 = getattr(conn, attr_name)
        with pytest.raises(AttributeError):

            # The code to be tested
            setattr(conn, attr_name, value)

        value2 = getattr(conn, attr_name)
        assert value2 == value1

    def test_no_recorder(self):  # pylint: disable=no-self-use
        """Test creation of wbem connection with specific parameters"""
        creds = ('myname', 'mypw')
        x509 = {'cert_file': 'mycertfile.crt', 'key_file': 'mykeyfile.key'}
        conn = WBEMConnection('http://localhost', creds,
                              default_namespace='root/blah',
                              x509=x509,
                              use_pull_operations=True,
                              stats_enabled=True)
        assert conn.url == 'http://localhost'
        assert conn.creds == creds
        assert conn.default_namespace == 'root/blah'
        assert conn.x509 == x509
        assert conn.stats_enabled is True
        assert conn.use_pull_operations is True

    def test_namespace_slashes_init(self):  # pylint: disable=no-self-use
        """Test stripping of leading and trailing slashes in default namespace
        of wbem connection when initializing"""
        conn = WBEMConnection('http://localhost', None,
                              default_namespace='//root/blah//')
        assert conn.default_namespace == 'root/blah'

    def test_namespace_slashes_set(self):  # pylint: disable=no-self-use
        """Test stripping of leading and trailing slashes in default namespace
        of wbem connection when setting the attribute"""
        conn = WBEMConnection('http://localhost', None,
                              default_namespace=None)
        with pytest.warns(DeprecationWarning):
            conn.default_namespace = '//root/blah//'
        assert conn.default_namespace == 'root/blah'

    def test_add_operation_recorder(self):  # pylint: disable=no-self-use
        """Test addition of an operation recorder"""
        conn = WBEMConnection('http://localhost')
        conn.add_operation_recorder(LogOperationRecorder('fake_conn_id'))
        # pylint: disable=protected-access
        assert len(conn._operation_recorders) == 1

    def test_add_operation_recorders(self):  # pylint: disable=no-self-use
        """Test addition of multiple operation recorders"""
        conn = WBEMConnection('http://localhost')
        conn.add_operation_recorder(LogOperationRecorder('fake_conn_id'))
        tcr_fn = 'blah.yaml'
        tcr_file = MyTestClientRecorder.open_file(tcr_fn, 'a')
        conn.add_operation_recorder(MyTestClientRecorder(tcr_file))
        # pylint: disable=protected-access
        assert len(conn._operation_recorders) == 2
        tcr_file.close()
        os.remove(tcr_fn)

    def test_add_same_twice(self):  # pylint: disable=no-self-use
        """
        Test addition of same recorder twice. This not allowed so
        generates exception
        """
        conn = WBEMConnection('http://localhost')
        conn.add_operation_recorder(LogOperationRecorder('fake_conn_id'))
        # pylint: disable=protected-access
        assert len(conn._operation_recorders) == 1

        with pytest.raises(ValueError):
            conn.add_operation_recorder(LogOperationRecorder('fake_conn_id'))


class TestGetRsltParams(object):
    """Test WBEMConnection._get_rslt_params method."""

    @pytest.mark.parametrize(
        'irval', [None, 'bla', [], {}])
    @pytest.mark.parametrize(
        # enumctxt input, eos input, expected_eos, exception_expected
        'ec, eos, eos_exp, exc_exp', [
            # following are successul returns
            [u'contextblah', u'False', False, False],   # normal enumctxt rtn
            ["", u'True', True, False],              # eos true, enmctxt empty
            [None, u'True', True, False],              # eos true, enmctxt None
            [u'contextblah', u'True', True, False],    # eos tru, cts with value
            # following are exceptions
            [None, None, None, True],       # fail,no values in eos or enumctxt
            [None, u'False', None, True],   # fail,no value in ec and eos False
        ]
    )
    # pylint: disable=no-self-use
    def test_with_params(self, irval, ec, eos, eos_exp, exc_exp):
        """
        Test combminations of IRETURNVALUE, EOS and EnumerationContext for
        both good and fail responses.
        """
        result = [
            (u'IRETURNVALUE', {}, irval),
            (u'EnumerationContext', None, ec),
            (u'EndOfSequence', None, eos)
        ]

        if exc_exp:
            with pytest.raises(CIMError) as exec_info:
                # pylint: disable=protected-access
                result = WBEMConnection._get_rslt_params(result, 'root/blah')

            exc = exec_info.value
            assert exc.status_code_name == 'CIM_ERR_INVALID_PARAMETER'

        else:
            # pylint: disable=protected-access
            result = WBEMConnection._get_rslt_params(result, 'root/blah')
            # the _get_rslt_params method sets context to None if eos True
            ecs = None if eos_exp is True else (ec, 'root/blah')
            assert result == (irval, eos_exp, ecs)

    # pylint: disable=no-self-use
    def test_missing(self):
        """
        Test both Enum context and EndOfSequence completely missing.
        Generates exception
        """
        result = [
            (u'IRETURNVALUE', {}, {})
        ]
        with pytest.raises(CIMError) as exec_info:
            # pylint: disable=protected-access
            result = WBEMConnection._get_rslt_params(result, 'namespace')

        exc = exec_info.value
        assert exc.status_code_name == 'CIM_ERR_INVALID_PARAMETER'



#!/usr/bin/env python

"""
Test exceptions module.
"""

from __future__ import absolute_import, print_function

import six
import pytest

from pywbem import Error, ConnectionError, AuthError, HTTPError, TimeoutError,\
    ParseError, VersionError, CIMError


def _assert_subscription(exc):
    """Test the exception defined by exc for required args"""
    # Access by subscription is only supported in Python 2:
    if six.PY2:
        assert exc[:] == exc.args
        for i, _ in enumerate(exc.args):
            assert exc[i] == exc.args[i]
            assert exc[i:] == exc.args[i:]
            assert exc[0:i] == exc.args[0:i]
    else:
        try:
            _ = exc[:]
        except TypeError:
            pass
        else:
            assert False, "Access by slice did not fail in Python 3"
        for i, _ in enumerate(exc.args):
            try:
                _ = exc[i]  # noqa: F841
            except TypeError:
                pass
            else:
                assert False, "Access by index did not fail in Python 3"


# The exception classes for which the simple test should be done:
@pytest.fixture(params=[
    Error, ConnectionError, AuthError, TimeoutError, ParseError, VersionError
], scope='module')
def simple_class(request):
    """Return request.param as defined by the fixture. Representing
       the exception classes.
    """
    return request.param


# The init arguments for the simple exception classes:
@pytest.fixture(params=[
    (),
    ('foo',),
    ('foo', 42),
], scope='module')
def simple_args(request):
    """Return request.param defined by the pytest.fixture
       representing arguments for the exception class
    """
    return request.param


def test_simple(simple_class, simple_args):
    """Test exceptions classes and arguments using the
      testfixtures.
    """
    # pylint: disable=redefined-outer-name

    exc = simple_class(*simple_args)

    # exc has no len()
    assert len(exc.args) == len(simple_args)
    for i, _ in enumerate(simple_args):
        assert exc.args[i] == simple_args[i]
        assert exc.args[i:] == simple_args[i:]
        assert exc.args[0:i] == simple_args[0:i]
        assert exc.args[:] == simple_args[:]

    _assert_subscription(exc)


# The init arguments for the HTTPError exception class:
@pytest.fixture(params=[
    # (status, reason, cimerror=None, cimdetails={})
    (200, 'OK'),
    (404, 'Not Found', 'instance xyz not found'),
    (404, 'Not Found', 'instance xyz not found', 'foo'),
], scope='module')
def httperror_args(request):
    """Returns init arguments for the HTTPError exception class as
       pytest.fixture
    """
    return request.param


def test_httperror(httperror_args):  # pylint: disable=redefined-outer-name
    """Test httperror arguments from test fixture"""
    exc = HTTPError(*httperror_args)

    assert exc.status == httperror_args[0]
    assert exc.reason == httperror_args[1]
    if len(httperror_args) < 3:
        assert exc.cimerror is None  # default value
    else:
        assert exc.cimerror == httperror_args[2]
    if len(httperror_args) < 4:
        assert exc.cimdetails == {}  # default value (set in init)
    else:
        assert exc.cimdetails == httperror_args[3]

    assert exc.args[0] == exc.status
    assert exc.args[1] == exc.reason
    assert exc.args[2] == exc.cimerror
    assert exc.args[3] == exc.cimdetails
    assert len(exc.args) == 4

    _assert_subscription(exc)


# The CIM status codes for the CIMError exception class:
@pytest.fixture(params=[
    # (code, name)
    # Note: We don't test the exact default description text.
    # Note: name = None means that the status code is invalid.
    (0, None),
    (1, 'CIM_ERR_FAILED'),
    (2, 'CIM_ERR_ACCESS_DENIED'),
    (3, 'CIM_ERR_INVALID_NAMESPACE'),
    (4, 'CIM_ERR_INVALID_PARAMETER'),
    (5, 'CIM_ERR_INVALID_CLASS'),
    (6, 'CIM_ERR_NOT_FOUND'),
    (7, 'CIM_ERR_NOT_SUPPORTED'),
    (8, 'CIM_ERR_CLASS_HAS_CHILDREN'),
    (9, 'CIM_ERR_CLASS_HAS_INSTANCES'),
    (10, 'CIM_ERR_INVALID_SUPERCLASS'),
    (11, 'CIM_ERR_ALREADY_EXISTS'),
    (12, 'CIM_ERR_NO_SUCH_PROPERTY'),
    (13, 'CIM_ERR_TYPE_MISMATCH'),
    (14, 'CIM_ERR_QUERY_LANGUAGE_NOT_SUPPORTED'),
    (15, 'CIM_ERR_INVALID_QUERY'),
    (16, 'CIM_ERR_METHOD_NOT_AVAILABLE'),
    (17, 'CIM_ERR_METHOD_NOT_FOUND'),
    (18, None),
    (19, None),
    (20, 'CIM_ERR_NAMESPACE_NOT_EMPTY'),
    (21, 'CIM_ERR_INVALID_ENUMERATION_CONTEXT'),
    (22, 'CIM_ERR_INVALID_OPERATION_TIMEOUT'),
    (23, 'CIM_ERR_PULL_HAS_BEEN_ABANDONED'),
    (24, 'CIM_ERR_PULL_CANNOT_BE_ABANDONED'),
    (25, 'CIM_ERR_FILTERED_ENUMERATION_NOT_SUPPORTED'),
    (26, 'CIM_ERR_CONTINUATION_ON_ERROR_NOT_SUPPORTED'),
    (27, 'CIM_ERR_SERVER_LIMITS_EXCEEDED'),
    (28, 'CIM_ERR_SERVER_IS_SHUTTING_DOWN'),
    (29, None),
    (30, None),
], scope='module')
def status_tuple(request):
    """pytest.fixture returns status codes for CIMError
       exception
    """
    return request.param


def test_cimerror_1(status_tuple):  # pylint: disable=redefined-outer-name
    """Test cimerror"""
    status_code = status_tuple[0]
    status_code_name = status_tuple[1]

    invalid_code_name = 'Invalid status code %s' % status_code
    invalid_code_desc = 'Invalid status code %s' % status_code

    exc = CIMError(status_code)

    assert exc.status_code == status_code
    if status_code_name is None:
        assert exc.status_description == invalid_code_desc
        assert exc.status_code_name == invalid_code_name
    else:
        assert isinstance(exc.status_description, six.string_types)
        assert exc.status_code_name == status_code_name

    assert exc.args[0] == exc.status_code
    assert exc.args[1] is None
    assert len(exc.args) == 2

    _assert_subscription(exc)


def test_cimerror_2(status_tuple):  # pylint: disable=redefined-outer-name
    """Test cimerror status tuple from date in status-tuple fixture"""

    status_code = status_tuple[0]
    status_code_name = status_tuple[1]

    invalid_code_name = 'Invalid status code %s' % status_code

    input_desc = 'foo'

    exc = CIMError(status_code, input_desc)

    assert exc.status_code == status_code
    assert exc.status_description == input_desc
    if status_code_name is None:
        assert exc.status_code_name == invalid_code_name
    else:
        assert exc.status_code_name == status_code_name

    assert exc.args[0] == exc.status_code
    assert exc.args[1] == input_desc
    assert len(exc.args) == 2

    _assert_subscription(exc)
